@@ -3,6 +3,7 @@
 ;; Copyright (C) 1993, 1994  Free Software Foundation, Inc.
 
 ;; Author: jka@ece.cmu.edu (Jay K. Adams)
+;; Maintainer: FSF
 ;; Keywords: data
 
 ;; This file is part of XEmacs.
@@ -22,7 +23,7 @@
 ;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 ;; 02111-1307, USA.
 
-;; Synched up with: FSF 19.34
+;; Synched up with: FSF 20.3.10.1
 
 ;;; Commentary: 
 
@@ -118,11 +119,11 @@ for `jka-compr-compression-info-list')."
   :type 'string
   :group 'jka-compr)
 
-(defvar jka-compr-use-shell t)
-
+(defvar jka-compr-use-shell
+  (not (memq system-type '(ms-dos windows-nt))))
 
 ;;; I have this defined so that .Z files are assumed to be in unix
-;;; compress format; and .gz files, in gzip format.
+;;; compress format; and .gz files, in gzip format, and .bz2 files in bzip fmt.
 (defcustom jka-compr-compression-info-list
   ;;[regexp
   ;; compr-message  compr-prog  compr-args
@@ -132,6 +133,10 @@ for `jka-compr-compression-info-list')."
      "compressing"    "compress"     ("-c")
      "uncompressing"  "uncompress"   ("-c")
      nil t]
+    ["\\.bz2\\(~\\|\\.~[0-9]+~\\)?\\'" ; XEmacs change
+     "bzip2ing"       "bzip2"        nil
+     "unbzip2ing"     "bzip2"        ("-d")
+     nil t]
     ["\\.tgz\\'"
      "zipping"        "gzip"         ("-c" "-q")
      "unzipping"      "gzip"         ("-c" "-q" "-d")
@@ -139,11 +144,7 @@ for `jka-compr-compression-info-list')."
     ["\\.gz\\(~\\|\\.~[0-9]+~\\)?\\'"
      "zipping"        "gzip"         ("-c" "-q")
      "unzipping"      "gzip"         ("-c" "-q" "-d")
-     t t]
-    ["\\.bz2\\(~\\|\\.~[0-9]+~\\)?\\'"
-     "bzipping"       "bzip2"        ("-f")
-     "unbzipping"     "bzip2"        ("-d")
-     nil t])
+     t t])
 
   "List of vectors that describe available compression techniques.
 Each element, which describes a compression technique, is a vector of
@@ -168,8 +169,8 @@ APPEND-FLAG EXTENSION], where:
 
    uncompress-args       is a list of args to pass to the uncompress program
 
-   append-flag           is non-nil if files compressed with this technique can
-                         be appended to without decompressing them first.
+   append-flag           is non-nil if this compression technique can be
+                         appended
 
    auto-mode flag        non-nil means strip the regexp from file names
                          before attempting to set the mode.
@@ -177,7 +178,7 @@ APPEND-FLAG EXTENSION], where:
 Because of the way `call-process' is defined, discarding the stderr output of
 a program adds the overhead of starting a shell each time the program is
 invoked."
-  :type '(repeat (vector :tag "Compression Technique"
+  :type '(repeat (vector :tag "Compression Technique" ; XEmacs change
 			 regexp
 			 (choice :tag "Compress Message"
 				 (string :format "%v")
@@ -195,7 +196,10 @@ invoked."
 
 (defvar jka-compr-mode-alist-additions
   (list (cons "\\.tgz\\'" 'tar-mode))
-  "A list of pairs to add to auto-mode-alist when jka-compr is installed.")
+  "A list of pairs to add to `auto-mode-alist' when jka-compr is installed.")
+
+;; List of all the elements we actually added to file-coding-system-alist.
+(defvar jka-compr-added-to-file-coding-system-alist nil)
 
 (defvar jka-compr-file-name-handler-entry
   nil
@@ -234,23 +238,22 @@ based on the filename itself and `jka-compr-compression-info-list'."
 
 
 (defun jka-compr-error (prog args infile message &optional errfile)
+
   (let ((errbuf (get-buffer-create " *jka-compr-error*"))
 	(curbuf (current-buffer)))
-    (set-buffer errbuf)
-    (widen) (erase-buffer)
-    (insert (format "Error while executing \"%s %s < %s\"\n\n"
-		     prog
-		     (mapconcat 'identity args " ")
-		     infile))
+    (with-current-buffer errbuf
+      (widen) (erase-buffer)
+      (insert (format "Error while executing \"%s %s < %s\"\n\n"
+		      prog
+		      (mapconcat 'identity args " ")
+		      infile))
 
-     (and errfile
-	  (insert-file-contents errfile))
-
-     (set-buffer curbuf)
+      (and errfile
+	   (insert-file-contents errfile)))
      (display-buffer errbuf))
-  (signal 'compression-error (list "Opening input file"
-				   (format "error %s" message)
-				   infile)))
+
+  (signal 'compression-error
+	  (list "Opening input file" (format "error %s" message) infile)))
 
 
 (defvar jka-compr-dd-program
@@ -279,6 +282,7 @@ to keep: LEN chars starting BEG chars from the beginning."
 			     ;; dd seems to be unreliable about
 			     ;; providing the last block.  So, always
 			     ;; read one more than you think you need.
+			     ;; XEmacs change -- de-ebolify
 			     (if count
 				 (concat "count=" (number-to-string (1+ count)))
 			       ""))))
@@ -306,7 +310,9 @@ to keep: LEN chars starting BEG chars from the beginning."
 (defun jka-compr-call-process (prog message infile output temp args)
   (if jka-compr-use-shell
 
-      (let ((err-file (jka-compr-make-temp-name)))
+      (let ((err-file (jka-compr-make-temp-name))
+	    (coding-system-for-read (or coding-system-for-read 'undecided))
+	    (coding-system-for-write 'no-conversion))
 	    
 	(unwind-protect
 
@@ -338,20 +344,18 @@ to keep: LEN chars starting BEG chars from the beginning."
 	(jka-compr-error prog args infile message))
 
     (and (stringp output)
-	 (let ((cbuf (current-buffer)))
-	   (set-buffer temp)
+	 (with-current-buffer temp
 	   (write-region (point-min) (point-max) output)
-	   (erase-buffer)
-	   (set-buffer cbuf)))))
+	   (erase-buffer)))))
 
 
 ;;; Support for temp files.  Much of this was inspired if not lifted
 ;;; from ange-ftp.
 
 (defcustom jka-compr-temp-name-template
-  "/tmp/jka-com"
+  (expand-file-name "jka-com" (temp-directory))
   "Prefix added to all temp files created by jka-compr.
-There should be no more than seven characters after the final `/'"
+There should be no more than seven characters after the final `/'."
   :type 'string
   :group 'jka-compr)
 
@@ -417,13 +421,17 @@ There should be no more than seven characters after the final `/'"
 		(compress-args (jka-compr-info-compress-args info))
 		(uncompress-args (jka-compr-info-uncompress-args info))
 		(base-name (file-name-nondirectory visit-file))
-		temp-file cbuf temp-buffer)
+		temp-file temp-buffer
+		;; we need to leave `last-coding-system-used' set to its
+		;; value after calling write-region the first time, so
+		;; that `basic-save-buffer' sees the right value.
+		;; XEmacs change: we don't have `last-coding-system-used'.
+		;; (coding-system-used last-coding-system-used)
+		)
 
-	    (setq cbuf (current-buffer)
-		  temp-buffer (get-buffer-create " *jka-compr-wr-temp*"))
-	    (set-buffer temp-buffer)
-	    (widen) (erase-buffer)
-	    (set-buffer cbuf)
+	    (setq temp-buffer (get-buffer-create " *jka-compr-wr-temp*"))
+	    (with-current-buffer temp-buffer
+	      (widen) (erase-buffer))
 
 	    (if (and append
 		     (not can-append)
@@ -441,25 +449,33 @@ There should be no more than seven characters after the final `/'"
 	     (message "%s %s..." compress-message base-name))
 	    
 	    (jka-compr-run-real-handler 'write-region
-					(list start end temp-file t 'dont lockname coding-system))
+					;; XEmacs change: add lockname, c-s
+					(list start end temp-file t 'dont
+					      lockname coding-system
+					      ))
 
-	    (jka-compr-call-process compress-program
-				    (concat compress-message
-					    " " base-name)
-				    temp-file
-				    temp-buffer
-				    nil
-				    compress-args)
-
-	    (set-buffer temp-buffer)
-	    (jka-compr-run-real-handler 'write-region
-					(list (point-min) (point-max)
-					      filename
-					      (and append can-append)
-					      'dont lockname 'binary))
-	    (erase-buffer)
-	    (set-buffer cbuf)
+	    ;; save value used by the real write-region
+	    ;; without any code conversion.
+	    (let ((coding-system-for-read 'no-conversion))
+	      (jka-compr-call-process compress-program
+				      (concat compress-message
+					      " " base-name)
+				      temp-file
+				      temp-buffer
+				      nil
+				      compress-args))
 
+	    (with-current-buffer temp-buffer
+	      (let ((coding-system-for-write 'no-conversion))
+		(if (memq system-type '(ms-dos windows-nt))
+		    (setq buffer-file-type t) )
+		(jka-compr-run-real-handler 'write-region
+					    (list (point-min) (point-max)
+						  filename
+						  (and append can-append) 'dont
+						  lockname coding-system
+						  ))
+		(erase-buffer)) )
 	    (jka-compr-delete-temp-file temp-file)
 
 	    (and
@@ -480,10 +496,17 @@ There should be no more than seven characters after the final `/'"
 		     (stringp visit))
 		 (message "Wrote %s" visit-file))
 
+	    ;; ensure `last-coding-system-used' has an appropriate value
+	    ;; XEmacs change: don't have `last-coding-system-used'
+	    ;; (setq last-coding-system-used coding-system-used)
+
 	    nil)
 	      
 	(jka-compr-run-real-handler 'write-region
-				    (list start end filename append visit lockname coding-system)))))
+				    (list start end filename append visit
+					  ;; XEmacs change
+					  lockname coding-system
+					  )))))
 
 
 (defun jka-compr-insert-file-contents (file &optional visit beg end replace)
@@ -506,8 +529,20 @@ There should be no more than seven characters after the final `/'"
 	      (local-copy
 	       (jka-compr-run-real-handler 'file-local-copy (list filename)))
 	      local-file
-	      size start)
-
+	      size start
+	      (coding-system-for-read
+	       (or coding-system-for-read
+		   'undecided)))
+		   ;; XEmacs change; skip multibyte-mode crap
+;		   (and (null enable-multibyte-characters)
+;			(or (auto-coding-alist-lookup
+;			     (jka-compr-byte-compiler-base-file-name file))
+;			    'raw-text))
+;		   (let ((coding (find-operation-coding-system
+;				  'insert-file-contents
+;				  (jka-compr-byte-compiler-base-file-name file))))
+;		     (and (consp coding) (car coding)))
+;		   'undecided)) )
 	  (setq local-file (or local-copy filename))
 
 	  (and
@@ -589,18 +624,22 @@ There should be no more than seven characters after the final `/'"
 	   (signal 'file-error
 		   (cons "Opening input file" (nth 2 notfound))))
 
+	  ;; This is done in insert-file-contents after we return.
+	  ;; That is a little weird, but better to go along with it now
+	  ;; than to change it now.
+
 	  ;; Run the functions that insert-file-contents would.
-	  (let ((p after-insert-file-functions)
-		(insval size))
-	    (while p
-	      (setq insval (funcall (car p) size))
-	      (if insval
-		  (progn
-		    (or (integerp insval)
-			(signal 'wrong-type-argument
-				(list 'integerp insval)))
-		    (setq size insval)))
-	      (setq p (cdr p))))
+;	  (let ((p after-insert-file-functions)
+;		(insval size))
+;	    (while p
+;	      (setq insval (funcall (car p) size))
+;	      (if insval
+;		  (progn
+;		    (or (integerp insval)
+;			(signal 'wrong-type-argument
+;				(list 'integerp insval)))
+;		    (setq size insval)))
+;	      (setq p (cdr p))))
 
 	  (list filename size))
 
@@ -623,42 +662,45 @@ There should be no more than seven characters after the final `/'"
 	      (temp-file (jka-compr-make-temp-name t))
 	      (temp-buffer (get-buffer-create " *jka-compr-flc-temp*"))
 	      (notfound nil)
-	      (cbuf (current-buffer))
 	      local-file)
 
 	  (setq local-file (or local-copy filename))
 
 	  (unwind-protect
 
-	      (progn
+	      (with-current-buffer temp-buffer
 		  
 		(and
 		 uncompress-message
 		 (message "%s %s..." uncompress-message base-name))
 
-		(set-buffer temp-buffer)
-		  
-		(jka-compr-call-process uncompress-program
-					(concat uncompress-message
-						" " base-name)
-					local-file
-					t
-					nil
-					uncompress-args)
+		;; Here we must read the output of uncompress program
+		;; and write it to TEMP-FILE without any code
+		;; conversion.  An appropriate code conversion (if
+		;; necessary) is done by the later I/O operation
+		;; (e.g. load).
+		(let ((coding-system-for-read 'no-conversion)
+		      (coding-system-for-write 'no-conversion))
+		  (jka-compr-call-process uncompress-program
+					  (concat uncompress-message
+						  " " base-name)
+					  local-file
+					  t
+					  nil
+					  uncompress-args)
 
-		(and
-		 uncompress-message
-		 (message "%s %s...done" uncompress-message base-name))
+		  (and
+		   uncompress-message
+		   (message "%s %s...done" uncompress-message base-name))
 
-		(write-region
-		 (point-min) (point-max) temp-file nil 'dont))
+		  (write-region
+		   (point-min) (point-max) temp-file nil 'dont)))
 
 	    (and
 	     local-copy
 	     (file-exists-p local-copy)
 	     (delete-file local-copy))
 
-	    (set-buffer cbuf)
 	    (kill-buffer temp-buffer))
 
 	  temp-file)
@@ -696,7 +738,7 @@ There should be no more than seven characters after the final `/'"
   (let ((info (jka-compr-get-compression-info file)))
     (if (and info (jka-compr-info-strip-extension info))
 	(save-match-data
-	 (substring file 0 (string-match (jka-compr-info-regexp info) file)))
+	  (substring file 0 (string-match (jka-compr-info-regexp info) file)))
       file)))
 
 (put 'write-region 'jka-compr 'jka-compr-write-region)
@@ -706,10 +748,15 @@ There should be no more than seven characters after the final `/'"
 (put 'byte-compiler-base-file-name 'jka-compr
      'jka-compr-byte-compiler-base-file-name)
 
+(defvar jka-compr-inhibit nil
+  "Non-nil means inhibit automatic uncompression temporarily.
+Lisp programs can bind this to t to do that.
+It is not recommended to set this variable permanently to anything but nil.")
+
 (defun jka-compr-handler (operation &rest args)
   (save-match-data
     (let ((jka-op (get operation 'jka-compr)))
-      (if jka-op
+      (if (and jka-op (not jka-compr-inhibit))
 	  (apply jka-op args)
 	(jka-compr-run-real-handler operation args)))))
 
@@ -771,7 +818,6 @@ saying whether the mode is now on or off."
 
     flag))
 
-
 (defun jka-compr-build-file-regexp ()
   (concat
    "\\("
@@ -781,6 +827,7 @@ saying whether the mode is now on or off."
     "\\)\\|\\(")
    "\\)"))
 
+;;; XEmacs:
 ;;;###autoload
 (defun jka-compr-install ()
   "Install jka-compr.
@@ -793,8 +840,20 @@ and `inhibit-first-line-modes-suffixes'."
   (setq file-name-handler-alist (cons jka-compr-file-name-handler-entry
 				      file-name-handler-alist))
 
+  (when (boundp 'file-coding-system-alist)
+    (setq jka-compr-added-to-file-coding-system-alist nil))
+
   (mapcar
    (function (lambda (x)
+	       ;; Don't do multibyte encoding on the compressed files.
+	       (when (boundp 'file-coding-system-alist)
+	       (let ((elt (cons (jka-compr-info-regexp x)
+                                '(no-conversion . no-conversion))))
+		 (setq file-coding-system-alist
+		       (cons elt file-coding-system-alist))
+		 (setq jka-compr-added-to-file-coding-system-alist
+		       (cons elt jka-compr-added-to-file-coding-system-alist)))
+	       )
 	       (and (jka-compr-info-strip-extension x)
 		    ;; Make entries in auto-mode-alist so that modes
 		    ;; are chosen right according to the file names
@@ -857,13 +916,43 @@ by `jka-compr-installed'."
 	  (setcdr last (cdr (cdr last)))
 	(setq last (cdr last))))
     
-    (setq auto-mode-alist (cdr ama))))
+    (setq auto-mode-alist (cdr ama)))
 
+  (when (boundp 'file-coding-system-alist)
+  (let* ((ama (cons nil file-coding-system-alist))
+	 (last ama)
+	 entry)
+
+    (while (cdr last)
+      (setq entry (car (cdr last)))
+      (if (member entry jka-compr-added-to-file-coding-system-alist)
+	  (setcdr last (cdr (cdr last)))
+	(setq last (cdr last))))
+
+    (setq file-coding-system-alist (cdr ama))))
+)
       
 (defun jka-compr-installed-p ()
   "Return non-nil if jka-compr is installed.
 The return value is the entry in `file-name-handler-alist' for jka-compr."
-  (rassq 'jka-compr-handler file-name-handler-alist))
+ 
+  (let ((fnha file-name-handler-alist)
+	(installed nil))
+
+    (while (and fnha (not installed))
+      (and (eq (cdr (car fnha)) 'jka-compr-handler)
+	   (setq installed (car fnha)))
+      (setq fnha (cdr fnha)))
+
+    installed))
+
+
+;;; Add the file I/O hook if it does not already exist.
+;;; Make sure that jka-compr-file-name-handler-entry is eq to the
+;;; entry for jka-compr in file-name-handler-alist.
+;; No no no no!
+;(and (jka-compr-installed-p)
+;     (jka-compr-uninstall))
 
 
 ;; No no no no!
