@@ -1,12 +1,12 @@
 ;;; rlogin.el --- remote login interface
 
-;; Copyright (C) 1992, 1993, 1994, 1995, 1997 Free Software Foundation, Inc.
+;; Copyright (C) 1992-1995, 1997, 1998 Free Software Foundation, Inc.
 
 ;; Author: Noah Friedman
-;; Maintainer: Noah Friedman <friedman@prep.ai.mit.edu>
+;; Maintainer: Noah Friedman <friedman@splode.com>
 ;; Keywords: unix, comm
 
-;; $Id: rlogin.el,v 1.40 1997/06/23 08:22:53 friedman Exp $
+;; $Id: rlogin.el,v 1.43 1998/09/11 01:22:53 friedman Exp $
 
 ;; This file is part of GNU Emacs.
 
@@ -47,7 +47,6 @@
   :group 'processes
   :group 'unix)
 
-
 (defcustom rlogin-program "rlogin"
   "*Name of program to invoke rlogin"
   :type 'string
@@ -78,8 +77,8 @@ If `nil', use a pipe (if pipes are supported on the local system).
 Generally it is better not to waste ptys on systems which have a static
 number of them.  On the other hand, some implementations of `rlogin' assume
 a pty is being used, and errors will result from using a pipe instead."
-  :type '(choice (const :tag "ptys" t)
-		 (const :tag "pipes" nil))
+  :type '(choice (const :tag "pipes" nil)
+		 (other :tag "ptys" t))
   :group 'rlogin)
 
 (defcustom rlogin-directory-tracking-mode 'local
@@ -101,7 +100,7 @@ simply setting this variable, since the function does the necessary
 re-synching of directories."
   :type '(choice (const :tag "off" nil)
 		 (const :tag "ftp" t)
-		 (const :tag "local" local))
+		 (other :tag "local" local))
   :group 'rlogin)
 
 (make-variable-buffer-local 'rlogin-directory-tracking-mode)
@@ -215,15 +214,19 @@ variable."
       ;; buffer from a previous exited process.
       (set-marker (process-mark proc) (point-max))
 
-      ;; comint-output-filter-functions is just like a hook, except that the
-      ;; functions in that list are passed arguments.  add-hook serves well
-      ;; enough for modifying it.
+      ;; comint-output-filter-functions is treated like a hook: it is
+      ;; processed via run-hooks or run-hooks-with-args in later versions
+      ;; of emacs.
       ;; comint-output-filter-functions should already have a
       ;; permanent-local property, at least in emacs 19.27 or later.
-      (if (fboundp 'make-local-hook)
-          (make-local-hook 'comint-output-filter-functions)
-        (make-local-variable 'comint-output-filter-functions))
-      (add-hook 'comint-output-filter-functions 'rlogin-carriage-filter)
+      (cond
+       ((fboundp 'make-local-hook)
+        (make-local-hook 'comint-output-filter-functions)
+        (add-hook 'comint-output-filter-functions 'rlogin-carriage-filter
+                  nil t))
+       (t
+        (make-local-variable 'comint-output-filter-functions)
+        (add-hook 'comint-output-filter-functions 'rlogin-carriage-filter)))
 
       (rlogin-mode)
 
@@ -320,7 +323,7 @@ local one share the same directories (through NFS)."
       (setq list (cons (substring line (match-beginning 0) (match-end 0))
                        list))
       (setq posn (match-end 0)))
-    (store-match-data (match-data))
+    (set-match-data (match-data))
     (nreverse list)))
 
 (defun rlogin-carriage-filter (string)
