@@ -1309,9 +1309,16 @@ Leaves the region wide."
 
 ;;; Patch it in.
 
-(defvar tar-regexp "\\.\\(tar\\|tgz\\|tar\\.gz\\)\\'"
-  "The regular expression used to identify tar file names.")
+;;;###autoload
+(defvar tar-regexp "\\.tar$"
+  "The regular expression used to identify tar file names.
+Note that this regular expression must not match compressed tar file
+names; if it does, tar-mode will attempt to parse the compressed tar
+file as an uncompressed tar file, which will generate an error.  This
+is not a problem, as other modules that handle compression will
+uncompress the buffer and call `tar-mode' appropriately.")
 
+;;;###autoload
 (setq auto-mode-alist
       (cons (cons tar-regexp 'tar-mode) auto-mode-alist))
 
